@@ -26,7 +26,6 @@ from cogniceptshell.common import permission_safe_docker_call
 from subprocess import DEVNULL
 from docker.types import LogConfig
 
-
 class AgentLifeCycle:
     """
     A class to manage agent and Cognicept's docker container lifecycle
@@ -65,8 +64,12 @@ class AgentLifeCycle:
     _docker_images["cgs_diagnostics_agent"] = "412284733352.dkr.ecr.ap-southeast-1.amazonaws.com/cognicept_diagnostics_agent:latest"
     _docker_images["cgs_diagnostics_ecs_api"] = "412284733352.dkr.ecr.ap-southeast-1.amazonaws.com/cognicept_diagnostics_api:latest"
     _docker_images["cgs_diagnostics_streamer_api"] = "412284733352.dkr.ecr.ap-southeast-1.amazonaws.com/cognicept_diagnostics_api:latest"
+    _docker_images["colab_master"] = "412284733352.dkr.ecr.ap-southeast-1.amazonaws.com/ros:master"
+    _docker_images["colab_description"] = "412284733352.dkr.ecr.ap-southeast-1.amazonaws.com/ros:colab-desc"
+    _docker_images["cgs_mission_sys"] = "412284733352.dkr.ecr.ap-southeast-1.amazonaws.com/mission_system:dev"
     _docker_images["orbitty"] = "412284733352.dkr.ecr.ap-southeast-1.amazonaws.com/orbitty:latest"
     _docker_images["cgs_bagger_server"] = "412284733352.dkr.ecr.ap-southeast-1.amazonaws.com/cognicept_rosbagger:latest"
+    _docker_images["health_aggregator"] = "412284733352.dkr.ecr.ap-southeast-1.amazonaws.com/health_aggregator:dev"
 
     def configure_containers(object, cfg):
         """
@@ -202,6 +205,36 @@ class AgentLifeCycle:
         except:
             print(bcolors.WARNING + "No event logs present." + bcolors.ENDC)
 
+    def check_docker_image_exists(object, args):
+        """
+        Checks if this docker image exists.
+        
+                Parameters:
+                        args (str): Input container names.
+        """
+        image_set = set()
+        missing_image = set()
+
+        # If list of agents is not specified, get all container_names
+        if(not hasattr(args, 'list') or len(args.list) == 0):
+            args.list = object._docker_container_names
+
+        # Based on args.list to get image(s)
+        for container_name in args.list:
+            image_set.add(object._docker_images[container_name])
+
+        success_flag = True
+        docker_client = permission_safe_docker_call(docker.from_env)
+        for image in image_set:
+            try:
+                image = docker_client.images.get(image)
+
+            except docker.errors.ImageNotFound:
+                missing_image.add(image) #Track image(s) that is/are missing
+                success_flag = False
+
+        return success_flag, missing_image
+
     def _parse_remote_intervention_agent_logs(object, logs):
         """
         Parses logs to find status of remote intervention agent. Prints status.
@@ -280,14 +313,20 @@ class AgentLifeCycle:
                 Returns:
                         result (bool): True if succeeded in attached mode, always True in detached mode
         """
-        if args.detach:
-            result = True
-            print("Running restart in detached mode")
-            p = Process(target=object._detached_restart, args=(args,))
-            p.start()
-        else:
-            result = object._restart_protocol(args)
+        success_flag, missing_images = object.check_docker_image_exists(args) #success_flag is a bool to check if all image(s) exist before running restart protocol
 
+        if success_flag:
+            if args.detach:
+                result = True
+                print("Running restart in detached mode")
+                p = Process(target=object._detached_restart, args=(args,))
+                p.start()
+            else:
+                result = object._restart_protocol(args)
+        else:
+            result = success_flag
+            print("Error: The following image(s) shown below cannot be found")
+            print(*missing_images, sep = "\n")
         return result
 
     def start(object, args):
@@ -393,7 +432,7 @@ class AgentLifeCycle:
             object._agent_run_options["remote_intervention_agent"]["devices"] = ["/dev/snd"]
         object._agent_run_options["kriya"] = object._agent_run_options["remote_intervention_agent"]
         object._agent_run_options["kriya_watchdog"] = {
-            "command": "python3 /home/watchdog.py", "volumes": {
+            "command": "python3 /home/watchdog.py", "network_mode": "host", "volumes": {
                 args.config.config_path + "kriya_logs/": {"bind": "/root/logs/", "mode": "rw"},
                 "/var/run/docker.sock/": {"bind": "/var/run/docker.sock/", "mode": "rw"}}}
 
@@ -522,39 +561,58 @@ class AgentLifeCycle:
             print(bcolors.FAIL + "DOCKER ERROR" + bcolors.ENDC)
         os.system("xhost -local:root")
 
-    def _update_protocol(object, args):
+    def cognicept_version_update(object):
+        current_version = pkg_resources.require("cognicept-shell")[0].version
+        package = 'cognicept-shell'
+        response = requests.get(f'https://pypi.org/pypi/{package}/json')
+        latest_version = response.json()['info']['version']
+        if latest_version != current_version:
+            print(f"{package} current version {current_version} - Installing Version {latest_version}")
+            os.system(f'pip3 install -q {package}=={latest_version}')
+            print(f'Installation {package} to version {latest_version}:'+bcolors.OKBLUE + " DONE" + bcolors.ENDC)
+        else:
+            print(f"{package} already in latest version={latest_version}")
+
+
+    def update_agents(object, args):
         """
-        Pulls docker images listed in `COG_AGENT_IMAGES`.
+        Starts the containers listed in `args.list` for the purpose of update.
 
                 Parameters:
                         args: populated argument namespace returned by `argparse.parse_args()`
                 Returns:
                         result (bool): True if succeeded
         """
-        # Attempt to login to ECR client
-        login_success = object._ecr_login(args)
 
-        # If login failed, return false
-        if login_success is False:
-            print("Your update credentials may have expired. Run `cognicept keyrotate` to refresh credentials and try `cognicept update` again.")
-            return False
+        # if list of agents is not specified, update all agents
+        if(not hasattr(args, 'list') or len(args.list) == 0): 
+        
+            images = set(object._docker_images.values())
+            # load extra images to update
+            if("COG_EXTRA_IMAGES" in args.config.config):
+                image_names = args.config.config["COG_EXTRA_IMAGES"].split(";")
+                if(len(image_names) > 0):
+                    images = images.union(set(image_names))
+            
+            N = len(images)
+            object.cognicept_version_update()
+            print("Info: This may take a while depending on your connection.")
+        # if list of agents is specified, update only that particular set of agents
+        else: 
+            images = set()
+            for container_name in args.list:
+                images.add(object._docker_images[container_name])
+            N = len(images)
+            print("Info: Update " + str(N) + " agent_image(s)." )
 
-        print("Info: This may take a while depending on your connection.")
-        images = set(object._docker_images.values())
-
-        # load extra images to update
-        if("COG_EXTRA_IMAGES" in args.config.config):
-            image_names = args.config.config["COG_EXTRA_IMAGES"].split(";")
-            if(len(image_names) > 0):
-                images = images.union(set(image_names))
-
-        N = len(images)
-        i = 0
+        i = 0 # For indexing
         success_flag = True
+
         for image_name in images:
             i = i + 1
-            image_name_short = image_name.replace(
-                "412284733352.dkr.ecr.ap-southeast-1.amazonaws.com/", "cognicept/")
+
+            image_name_short = image_name.replace("412284733352.dkr.ecr.ap-southeast-1.amazonaws.com/", "cognicept/")
+
             try:
                 for status in object.docker_client.pull(image_name, stream=True, decode=True):
                     if("progress" not in status):
@@ -575,18 +633,37 @@ class AgentLifeCycle:
                             print('{:{trm_sz}.{trm_sz}}'.format(
                                 status, trm_sz=80), end="\r", flush=True)
                 print("[" + str(i) + "/" + str(N) + "] " + image_name_short +
-                      " - " + bcolors.OKBLUE + "OK" + bcolors.ENDC + "\033[K")
+                    " - " + bcolors.OKBLUE + "OK" + bcolors.ENDC + "\033[K")
             except docker.errors.ImageNotFound:
                 print("[" + str(i) + "/" + str(N) + "] " + image_name_short +
-                      " - " + bcolors.FAIL + "FAILED" + bcolors.ENDC + "\033[K")
+                    " - " + bcolors.FAIL + "FAILED" + bcolors.ENDC + "\033[K")
                 success_flag = False
             except:
                 print("[" + str(i) + "/" + str(N) + "] " + image_name_short +
-                      " - " + bcolors.FAIL + "FAILED" + bcolors.ENDC + "\033[K")
+                    " - " + bcolors.FAIL + "FAILED" + bcolors.ENDC + "\033[K")
                 success_flag = False
-
         print("Info: Run `cognicept restart` to redeploy updated agents.")
+        
         return success_flag
+        
+    def _update_protocol(object, args):
+        """
+        Pulls docker images listed in `COG_AGENT_IMAGES`.
+
+                Parameters:
+                        args: populated argument namespace returned by `argparse.parse_args()`
+                Returns:
+                        result (bool): True if succeeded
+        """
+        # Attempt to login to ECR client
+        login_success = object._ecr_login(args)
+
+        # If login failed, return false
+        if login_success is False:
+            print("Your update credentials may have expired. Run `cognicept keyrotate` to refresh credentials and try `cognicept update` again.")
+            return False
+
+        return object.update_agents(args)
 
     def _detached_update(object, args):
         """
