@@ -43,6 +43,8 @@ class Configuration:
         Function to setup ssh access for the host machine given user input.
     is_ssh_enabled(args):
         Checks and returns value of `COG_ENABLE_SSH_KEY_AUTH` as bool.
+    is_audio_enabled(args):
+        Checks and returns value of `COG_ENABLE_AUDIO` as bool
     cognicept_key_rotate:
         Rotates AWS temporary keys retriving them from Cognicept API.
     init_config(args):
@@ -356,13 +358,27 @@ class Configuration:
                 Parameters:
                         None
                 Returns:
-                        ssh_enabled (bool): True if ssh is enables
+                        ssh_enabled (bool): True if ssh is enabled
         """
         if(not "COG_ENABLE_SSH_KEY_AUTH" in object.config):
             return False
 
         return object.config["COG_ENABLE_SSH_KEY_AUTH"]
 
+    def is_audio_enabled(object):
+        """
+        Checks and returns value of `COG_ENABLE_AUDIO` as bool
+
+                Parameters:
+                        None
+                Returns:
+                        sound enabled (bool): True if sound is enabled
+        """
+        if(not "COG_ENABLE_AUDIO" in object.config):
+            return False
+
+        return object.config["COG_ENABLE_AUDIO"]
+    
 
     def fetch_aws_keys(object):
         """
